#include "flash.h"

#define FLASH_KEY1 ((uint32_t)0x45670123)
#define FLASH_KEY2 ((uint32_t)0xCDEF89AB)

#define PAGE_12	(0x08000000UL+12UL*1024)
#define PAGE_13	(0x08000000UL+13UL*1024)
#define PAGE_14	(0x08000000UL+14UL*1024)
#define PAGE_15	(0x08000000UL+15UL*1024)
#define PAGE_1	(0x08000000UL+30UL*1024)//30
#define PAGE_2	(0x08000000UL+31UL*1024)//31
//0x0800 3C00 


uint32_t ParamFlashError;
//void flash_unlock(void);
//void flash_lock(void);
void flash_erase_page(uint32_t address);
void flash_write(uint32_t address,uint32_t data);



static void flash_unlock(void) {
	FLASH->KEYR = FLASH_KEY1;
	FLASH->KEYR = FLASH_KEY2;
}

static void flash_lock(void) {
	FLASH->CR |= FLASH_CR_LOCK;
}


//   .      
//       .
void flash_erase_page(uint32_t address) {
  FLASH->CR|= FLASH_CR_PER; //    
  FLASH->AR = address; //   
  FLASH->CR|= FLASH_CR_STRT; //   
  while(FLASH->SR & FLASH_SR_BSY);//   . 
  FLASH->CR&= ~FLASH_CR_PER; //  
}

void flash_write(uint32_t address,uint32_t data) {
  FLASH->CR |= FLASH_CR_PG; //  
  while(FLASH->SR & FLASH_SR_BSY); //    

  *(__IO uint16_t*)address = (uint16_t)data; //  2 
  while(FLASH->SR & FLASH_SR_BSY);

  *(__IO uint16_t*)(address+2) = (uint16_t)(data>>16); //  2 
  while(FLASH->SR & FLASH_SR_BSY);

  FLASH->CR &= ~(FLASH_CR_PG); //  
}

void flash_struct_save(uint32_t *Data, uint8_t *D, uint32_t Size){
uint32_t j, Adr, crc=0;
	
	flash_unlock();

	if ( (*(__IO uint32_t*)PAGE_1)!=0xFFFFFFFF ){
		Adr=PAGE_2;
		flash_erase_page(PAGE_1);
		}
		else{
		Adr=PAGE_1;
		flash_erase_page(PAGE_2);
		}
	
	for(j=0; j<Size-sizeof(uint32_t); j++){
		crc+=D[j];
		}
	Data[Size/sizeof(uint32_t)-1]=crc;
		
	for(j=0; j<(Size/sizeof(uint32_t)); j++){
		flash_write(Adr+j*sizeof(uint32_t),*Data);
		Data++;
		}
		
	flash_lock();
	ParamFlashError=0;
}

//   size/sizeof(uint32_t)
void flash_param_read(uint32_t *Data, uint8_t *D, uint32_t Size){
	uint32_t j, crc=0;
	__IO uint32_t *p;
	//p=(__IO uint32_t*)PAGE_2;
	
	if ( (*(__IO uint32_t*)PAGE_1)==0xFFFFFFFF ) p=(__IO uint32_t*)PAGE_2;
	else
	if ( (*(__IO uint32_t*)PAGE_2)==0xFFFFFFFF ) p=(__IO uint32_t*)PAGE_1;
	else {
		ParamFlashError=1;
		return;
		}

	for(j=0; j < Size/sizeof(uint32_t); j++){
		Data[j]=p[j];
		//p++;
		}

	for(j=0; j<Size-sizeof(uint32_t); j++){
		crc+=D[j];
		}
		
	if (Data[Size/sizeof(uint32_t)-1] == crc) ParamFlashError=0; else ParamFlashError=1;
}


/*
FLASH_CR:
PG -         
PER -   .        -   FLASH_AR.            . 
MER -    .  , .
STRT -    ( ,         ). 
LOCK -           .
FLASH_SR:
BSY


void flash_write(uint32_t address,uint32_t data) {
  FLASH->CR |= FLASH_CR_PG; //  
  while(FLASH->SR & FLASH_SR_BSY); //    

  *(__IO uint16_t*)address = (uint16_t)data; //  2 
  while(FLASH->SR & FLASH_SR_BSY);

  *(__IO uint16_t*)(address+2) = (uint16_t)(data>>16); //  2 
  while(FLASH->SR & FLASH_SR_BSY);

  FLASH->CR &= ~(FLASH_CR_PG); //  
}

void flash_struct_save(uint32_t *Data, uint32_t Size){
	uint32_t j;
	uint32_t Adr;
	flash_unlock();

	if ( (*(__IO uint32_t*)PAGE_14)!=0xFFFFFFFF ){
		Adr=PAGE_15;
		flash_erase_page(PAGE_14);
		}
		else{
		Adr=PAGE_14;
		flash_erase_page(PAGE_15);
		}
	
	for(j=0; j<(Size/sizeof(uint32_t)); j++){
		flash_write(Adr+j*sizeof(uint32_t),*Data);
		Data++;
		}
		
	flash_lock();
	ParamFlashError=0;
}
*/

