// ***********************************************************
// Project: lcd HD44780 4-bit
// Author: StartCd.narod.ru
// : 1.41
// for stm32
//   LCD   
// ***********************************************************
#include <stm32f10x.h>
//#include "stdint.h"

#include "lph.h"
#include "commonfile.h"
#include "gpio.h"
#include "timers.h"

static void lcd_4bus_io(uint8_t data){
	off(DB7_PIN);
	off(DB6_PIN);
	off(DB5_PIN);
	off(DB4_PIN); 

	if (data&BIT(7)) on(DB7_PIN);
	if (data&BIT(6)) on(DB6_PIN);
	if (data&BIT(5)) on(DB5_PIN);
	if (data&BIT(4)) on(DB4_PIN);
	
	on(E_PIN);
	delay_us(3);
	off(E_PIN);
}

void lcd_4bus_write(uint8_t data)	//   4  
{	//  
	
	//DB7_CFG_R &= ~_CLR_MODE_CNF(DB7);//DRIVER(DB7,OUT);
	//DB7_CFG_R |= _OUT_2MHZ_PP(DB7);
	pmode(DB7_PIN, PIN_CFG_OPP_2MHZ);
	pmode(DB6_PIN, PIN_CFG_OPP_2MHZ);
	pmode(DB5_PIN, PIN_CFG_OPP_2MHZ);
	pmode(DB4_PIN, PIN_CFG_OPP_2MHZ);
	
	lcd_4bus_io(data);
	lcd_4bus_io(data<<4);
	
	off(DB7_PIN);
	off(DB6_PIN);
	off(DB5_PIN);
	off(DB4_PIN); 

	pmode(DB7_PIN, PIN_CFG_IP);//PIN_CFG_IF
	pmode(DB6_PIN, PIN_CFG_IP);
	pmode(DB5_PIN, PIN_CFG_IP);
	pmode(DB4_PIN, PIN_CFG_IP);
}

void lcd_write_data(uint8_t data){
	//write data to CGRAM or DDRAM
	delay_us(50);
	on(RS_PIN);//ON(RS);//RS = 1 -   (DR). RW = 0 - 
	lcd_4bus_write(data);
	off(RS_PIN);//OFF(RS);
}



void lcd_instructions_set(uint8_t address){
	//   -> address|( , cgram  ddram)
	delay_us(50);
	lcd_4bus_write(address);
}

void lcd_set_address(uint8_t x, uint8_t y){
	x+=SET_DDRAM_ADDRESS;
	if (y) x+=0x40;
	lcd_instructions_set(x);
}

void lcd_init(void){

//	delay_ms(10);
	
	//---------------------
	pmode(E_PIN,  PIN_CFG_OPP_2MHZ);
	pmode(RS_PIN, PIN_CFG_OPP_2MHZ);

	//-------------------------------
	pmode(DB7_PIN, PIN_CFG_OPP_2MHZ);
	pmode(DB6_PIN, PIN_CFG_OPP_2MHZ);
	pmode(DB5_PIN, PIN_CFG_OPP_2MHZ);
	pmode(DB4_PIN, PIN_CFG_OPP_2MHZ);
	//------------------------------
	
	off(E_PIN);
	off(RS_PIN);
	
	off(DB7_PIN);
	off(DB6_PIN);
	off(DB5_PIN);
	off(DB4_PIN);
	//---
	delay_ms(100);	//   20 (   5)

	
	on(E_PIN);//ON(E);	//-----------1--------------
	on(DB5_PIN);//ON(DB5);		//  $30 
	on(DB4_PIN);//ON(DB4);	
	delay_us(3);
	off(E_PIN);//OFF(E);			//-------------------------
		
	delay_ms(5);	//  2ms
	
	on(E_PIN);//ON(E);			//-----------2--------------
	//ON(DB5);		//  $30 
	//ON(DB4);
	delay_us(3);
	off(E_PIN);//OFF(E);			//-------------------------
	
	delay_us(110);//  100us
	
	on(E_PIN);//ON(E);			//-----------3--------------
	//ON(DB5);		//  $30.          
	//ON(DB4);		//     (       8-  ) 
	delay_us(3);	//3   .
	off(E_PIN);//OFF(E);			//-------------------------
	
	delay_us(50);	//40us
	
	off(DB4_PIN);//OFF(DB4);		//D/L:  ,    : 0 - 4 , 1 - 8 .
	on(E_PIN);;//ON(E);			 
	//ON(DB5);
	delay_us(3);		
	off(E_PIN);//OFF(E);
	
	lcd_instructions_set(0x28);//0x28 - 2 , 5  8 . (0x2C - 2  510)
	lcd_instructions_set(0x08);// : D e - ,
	lcd_instructions_set(0x06);//    =1 - .
	lcd_instructions_set(CLEAR_DISPLAY);
	delay_ms(5);//2m
	lcd_instructions_set(CURSOR_OFF);//   
//SystemCoreClock=0;
}

/*
static void lcd_4bus_io(uint8_t data){
	DB7_GPIO->BRR=BIT(DB7);//OFF(DB7);
	DB6_GPIO->BRR=BIT(DB6);//OFF(DB6);
	DB5_GPIO->BRR=BIT(DB5);//OFF(DB5);
	DB4_GPIO->BRR=BIT(DB4);//OFF(DB4); 

	if (data&BIT(7)) DB7_GPIO->BSRR=BIT(DB7);//ON(DB7);
	if (data&BIT(6)) DB6_GPIO->BSRR=BIT(DB6);//ON(DB6);
	if (data&BIT(5)) DB5_GPIO->BSRR=BIT(DB5);//ON(DB5);
	if (data&BIT(4)) DB4_GPIO->BSRR=BIT(DB4);//ON(DB4);
	E_GPIO->BSRR=BIT(E);//ON(E);
	delay_us(TIME_X(3));
	E_GPIO->BRR=BIT(E);//OFF(E);
}

void lcd_4bus_write(uint8_t data)	//   4  
{	//  
	DB7_CFG_R &= ~_CLR_MODE_CNF(DB7);//DRIVER(DB7,OUT);
	DB6_CFG_R &= ~_CLR_MODE_CNF(DB6);
	DB5_CFG_R &= ~_CLR_MODE_CNF(DB5);
	DB4_CFG_R &= ~_CLR_MODE_CNF(DB4);

	DB7_CFG_R |= _OUT_2MHZ_PP(DB7);
	DB6_CFG_R |= _OUT_2MHZ_PP(DB6);
	DB5_CFG_R |= _OUT_2MHZ_PP(DB5);
	DB4_CFG_R |= _OUT_2MHZ_PP(DB4);
	
	lcd_4bus_io(data);
	lcd_4bus_io(data<<4);
	
	DB7_GPIO->BRR=BIT(DB7);//OFF(DB7);
	DB6_GPIO->BRR=BIT(DB6);//OFF(DB6);
	DB5_GPIO->BRR=BIT(DB5);//OFF(DB5);
	DB4_GPIO->BRR=BIT(DB4);//OFF(DB4); 

	
	DB7_CFG_R &= ~_CLR_MODE_CNF(DB7);
	DB6_CFG_R &= ~_CLR_MODE_CNF(DB6);
	DB5_CFG_R &= ~_CLR_MODE_CNF(DB5);
	DB4_CFG_R &= ~_CLR_MODE_CNF(DB4);
	
	DB7_CFG_R |= _GPIO_IN_FLOAT(DB7);//DRIVER(DB7,IN);
	DB6_CFG_R |= _GPIO_IN_FLOAT(DB6);
	DB5_CFG_R |= _GPIO_IN_FLOAT(DB5);
	DB4_CFG_R |= _GPIO_IN_FLOAT(DB4);
}

uint8_t lcd_read_byte(void){//   4   
uint8_t byte=0;
	//OFF(DB7);
	//OFF(DB6);
	//OFF(DB5);
	//OFF(DB4);
	//DRIVER(DB7,IN);
	//DRIVER(DB6,IN);
	//DRIVER(DB5,IN);
	//DRIVER(DB4,IN);
	
	DB7_GPIO->BRR=BIT(DB7);//OFF(DB7);
	DB6_GPIO->BRR=BIT(DB6);//OFF(DB6);
	DB5_GPIO->BRR=BIT(DB5);//OFF(DB5);
	DB4_GPIO->BRR=BIT(DB4);//OFF(DB4);
	
	DB7_CFG_R &= ~_CLR_MODE_CNF(DB7);
	DB6_CFG_R &= ~_CLR_MODE_CNF(DB6);
	DB5_CFG_R &= ~_CLR_MODE_CNF(DB5);
	DB4_CFG_R &= ~_CLR_MODE_CNF(DB4);
	
	DB7_CFG_R |= _GPIO_IN_FLOAT(DB7);//DRIVER(DB7,IN);
	DB6_CFG_R |= _GPIO_IN_FLOAT(DB6);
	DB5_CFG_R |= _GPIO_IN_FLOAT(DB5);
	DB4_CFG_R |= _GPIO_IN_FLOAT(DB4);
	
	RS_GPIO->BRR=BIT(RS);//OFF(RS);
	RW_GPIO->BSRR=BIT(RW);//ON(RW);

	E_GPIO->BSRR=BIT(E);//ON(E);
	delay_us(TIME_X(3));
	if ( DB7_GPIO->IDR & BIT(DB7) ) byte|=BIT(7);//if ( ACTIVE(DB7) ) byte|=BIT(7);
	if ( DB6_GPIO->IDR & BIT(DB6) ) byte|=BIT(6);
	if ( DB5_GPIO->IDR & BIT(DB5) ) byte|=BIT(5);
	if ( DB4_GPIO->IDR & BIT(DB4) ) byte|=BIT(4);
	E_GPIO->BRR=BIT(E);//OFF(E);
	
	delay_us(TIME_X(1));

	E_GPIO->BSRR=BIT(E);//ON(E);
	delay_us(TIME_X(3));//3
	if ( DB7_GPIO->IDR & BIT(DB7) ) byte|=BIT(3);
	if ( DB6_GPIO->IDR & BIT(DB6) ) byte|=BIT(2);
	if ( DB5_GPIO->IDR & BIT(DB5) ) byte|=BIT(1);
	if ( DB4_GPIO->IDR & BIT(DB4) ) byte|=BIT(0);
	E_GPIO->BRR=BIT(E);//OFF(E);

	RW_GPIO->BRR=BIT(RW);//OFF(RW);
	return byte;
	//  , RS=RW=0,  DB3..0 = 0    Z  
}

uint8_t read_busy_address(void){
uint8_t byte;

	while( (uint8_t) ((byte=lcd_read_byte()) & BUSY_FLAG) );//   BUSY_FLAG
	return byte;
	//   AC, RS=RW=0,  DB3..0 = 0    Z  
}

void lcd_write_data(uint8_t data){
	//write data to CGRAM or DDRAM
	read_busy_address();
	RS_GPIO->BSRR=BIT(RS);//ON(RS);//RS = 1 -   (DR). RW = 0 - 
	lcd_4bus_write(data);
	RS_GPIO->BRR=BIT(RS);//OFF(RS);
}



void lcd_instructions_set(uint8_t address){
	//   -> address|( , cgram  ddram)
	read_busy_address();
	lcd_4bus_write(address);
}

void lcd_set_address(uint8_t x, uint8_t y){
	x+=SET_DDRAM_ADDRESS;
	if (y) x+=0x40;
	lcd_instructions_set(x);
}

//void lcd_set_address(uint8_t x, uint8_t y){
//x|=set_ddram_address;
////if (y==0) x|=0x00;
//if (y==1) x+=0x40;
//if (y==2) x+=0x14;
//if (y==3) x+=0x54;
//lcd_instructions_set(x);
//}

void lcd_init(void){

	__NOP();
	__NOP();
	__NOP();
	__NOP();
	__NOP();
	__NOP();
	__NOP();
	
	//---------------------
	E_CFG_R  &= ~_CLR_MODE_CNF(E);//  
	RW_CFG_R &= ~_CLR_MODE_CNF(RW);
	RS_CFG_R &= ~_CLR_MODE_CNF(RS);
	
	E_CFG_R  |= _OUT_2MHZ_PP(E);
	RW_CFG_R |= _OUT_2MHZ_PP(RW);
	RS_CFG_R |= _OUT_2MHZ_PP(RS);
	//-------------------------------
	DB7_CFG_R &= ~_CLR_MODE_CNF(DB7);//DRIVER(DB7,OUT);
	DB6_CFG_R &= ~_CLR_MODE_CNF(DB6);
	DB5_CFG_R &= ~_CLR_MODE_CNF(DB5);
	DB4_CFG_R &= ~_CLR_MODE_CNF(DB4);

	DB7_CFG_R |= _OUT_2MHZ_PP(DB7);
	DB6_CFG_R |= _OUT_2MHZ_PP(DB6);
	DB5_CFG_R |= _OUT_2MHZ_PP(DB5);
	DB4_CFG_R |= _OUT_2MHZ_PP(DB4);
	//------------------------------
	
	E_GPIO->BRR =BIT(E);//OFF();
	RW_GPIO->BRR=BIT(RW);//OFF();
	RS_GPIO->BRR=BIT(RS);//OFF();
	
	DB7_GPIO->BRR=BIT(DB7);//OFF(DB7);
	DB6_GPIO->BRR=BIT(DB6);//OFF(DB6);
	DB5_GPIO->BRR=BIT(DB5);//OFF(DB5);
	DB4_GPIO->BRR=BIT(DB4);//OFF(DB4); 
	//---
	delay_ms(TIME_X(50));	//   20 (   5)

	
	E_GPIO->BSRR =BIT(E);//ON(E);	//-----------1--------------
	DB5_GPIO->BSRR=BIT(DB5);//ON(DB5);		//  $30 
	DB4_GPIO->BSRR=BIT(DB4);//ON(DB4);	
	E_GPIO->BRR =BIT(E);//OFF(E);			//-------------------------
		
	delay_ms(TIME_X(5));	//  2ms
	
	E_GPIO->BSRR =BIT(E);//ON(E);			//-----------2--------------
	//ON(DB5);		//  $30 
	//ON(DB4);
	delay_us(TIME_X(3));
	E_GPIO->BRR =BIT(E);//OFF(E);			//-------------------------
	
	delay_us(TIME_X(110));//  100us
	
	E_GPIO->BSRR =BIT(E);//ON(E);			//-----------3--------------
	//ON(DB5);		//  $30.          
	//ON(DB4);		//     (       8-  ) 
	delay_us(TIME_X(3));	//3   .
	E_GPIO->BRR =BIT(E);//OFF(E);			//-------------------------
	
	delay_us(TIME_X(50));	//40us
	
	DB4_GPIO->BRR=BIT(DB4);//OFF(DB4);		//D/L:  ,    : 0 - 4 , 1 - 8 .
	E_GPIO->BSRR =BIT(E);//ON(E);			 
	//ON(DB5);
	delay_us(TIME_X(3));		
	E_GPIO->BRR =BIT(E);//OFF(E);
	
	lcd_instructions_set(0x28);//0x28 - 2 , 5  8 . (0x2C - 2  510)
	lcd_instructions_set(0x08);// : D e - ,
	lcd_instructions_set(0x06);//    =1 - .
	lcd_instructions_set(CLEAR_DISPLAY);
	delay_ms(TIME_X(5));//2m
	lcd_instructions_set(CURSOR_OFF);//   
//SystemCoreClock=0;
}
*/

